/*************************************************************************
*
* BG Office Project
* Copyright (C) 2000-2004 Radostin Radnev <radnev@yahoo.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*************************************************************************/


/*
* $Id: main.cpp 752 2019-01-29 19:20:42Z radnev $
*/

#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include "config.h"
#include "properties.h"
#include "translator_manager.h"


#ifdef DATA_DIR_CONFIG
const char DATA_DIR[] = DATA_DIR_CONFIG;
#else
const char DATA_DIR[] = "/usr/local/share/bgoffice/";
#endif

const char OPTION_LAST_USED_DICTIONARY[] = "LastUsedDictionary";

const char ABOUT_MESSAGE[] =
	"bgOffice Assistant version %s, Copyright (C) 2000-2004, Radostin Radnev\n"
	"\n"
	"Home page: http://bgoffice.sourceforge.net\n"
	"\n"
	"bgOffice Assistant comes with ABSOLUTELY NO WARRANTY. This is free\n"
	"software, and you are welcome to redistribute it under certain conditions.\n"
	"For details read file COPYING that comes with this package.\n\n";

const char MISSING_DATA[] =
	"Cannot find dictionary files.\n"
	"Please look at README for additional information and help or\n"
	"visit http://bgoffice.sourceforge.net to download dictionaries.\n\n";

const char HELP_MESSAGE[] =
	"Enter a word and press <Enter> to translate it.\n"
	"Type \\h to list available commands.\n\n";

const char COMMANDS[] =
	"\\h     Shows this information.\n"
	"\\q     Quits the application.\n"
	"\\l     Lists installed dictionaries.\n"
	"\\s     Shows active dictionary.\n"
	"\\s n   Sets given dictionary number as active.\n"
	"\\a     Lists abbreviations.\n\n";

const char PROMPT[] = "word: ";
const char ACTIVE_DICTIONARY[] = "Active dictionary is \"%s\".\n\n";
const char FOUND[] = "*** FOUND ***\n\n";
const char SIMILAR[] = "*** SIMILAR FOUND ***\n\n";
const char NOT_FOUND[] = "*** NOT FOUND ***\n\n";
const char MISSING_ABRR_FILE[] = "*** Abbreviations file is missing ***\n\n";

int main(int argc, char **argv) {

	Properties *prop = new Properties(getenv("HOME"), ".cbgoffice");
	char *path = prop->getString("DataDir", DATA_DIR);
	char *dataDir = new char[strlen(path) + 2];
	strcpy(dataDir, path);
	if ((dataDir[0] != '\0') && (dataDir[strlen(dataDir) - 1] != '/')) {
		strcat(dataDir, "/");
	}

	if ((argc > 1) && (strcmp(argv[1], "--dump-data-dir") == 0)) {
		printf("%s\n", dataDir);
		return 0;
	}

	printf(ABOUT_MESSAGE, VERSION);

	TranslatorManager *dictionary = new TranslatorManager(dataDir);
	if (!dictionary->init(true)) {
		printf(MISSING_DATA);
		return 1;
	}

	dictionary->setSeparateMeanings(false);
	dictionary->setTryWithoutSuffix(true);
	dictionary->setAdvancedSearchWholeWord(true);
	dictionary->setAdvancedSearchExactPhrase(false);
	dictionary->setAdvancedSearchHighlight(false);
	dictionary->setBoldDecoration(false);
	dictionary->setHTMLOutput(false);
	dictionary->setCurrentDictionary(prop->getInt(OPTION_LAST_USED_DICTIONARY, dictionary->getCurrentDictionary()));

	printf(HELP_MESSAGE);

	int MAX_WORD_LEN = 64;
	char word[MAX_WORD_LEN];
	char *Translation = NULL;
	bool found = true;

	while (true) {
		printf(PROMPT);
		fflush(stdin);
		fgets(word, MAX_WORD_LEN, stdin);

		if (word[0] != '\\') {
			found = dictionary->findWord(word, &Translation);
			if (Translation[0] != '\0') {
				printf(Translation);
				printf("\n\n");
				if (found) {
					if (!strncmp(dictionary->getWord(), word, strlen(dictionary->getWord()))) {
						printf(FOUND);
					} else {
						printf(SIMILAR);
					}
				} else {
					printf(NOT_FOUND);
				}
			}
		} else if (!strncasecmp("\\h", word, 2)) {
			printf(COMMANDS);
		} else if (!strncasecmp("\\q", word, 2)) {
			break;
		} else if (!strncasecmp("\\l", word, 2)) {
			for (int i = 0; i < dictionary->sizeOfDictionaries(); i++) {
				printf("%d. %s\n", i + 1, (dictionary->getDictionary(i)).name.c_str());
			}
			printf("\n");
		} else if (!strncasecmp("\\s", word, 2)) {
			int i = 0;
			sscanf(&word[3], "%d", &i);
			dictionary->setCurrentDictionary(i - 1);
			printf(ACTIVE_DICTIONARY, (dictionary->getDictionary(dictionary->getCurrentDictionary())).name.c_str());
		} else if (!strncasecmp("\\a", word, 2)) {
			Translation = dictionary->getAbbreviations();
			if (Translation[0] != '\0') {
				printf(Translation);
			} else {
				printf(MISSING_ABRR_FILE);
			}
		}
    }

	prop->setInt(OPTION_LAST_USED_DICTIONARY, dictionary->getCurrentDictionary());
	delete dictionary;
	return 0;

}
